require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/request/request/request_attributes'
require 'uu/request/request/request_set_attributes'
require 'uu/os/rest/paging_criteria'

module UU
  module Request
    module Request
      extend self

      # Service path
      PATH = 'ues/request/UESRequest'
      # This command returns attributes representing the request specified by the
      # requestUri parameter. The command does not change the state of the request
      # in the system.
      #
      # @param [UU::OS::UESURI,String] request_uri UESURI of a request whose attributes are returned
      #
      # @return [RequestAttributes] DTO containing the request attributes
      def get_attributes(request_uri)
        svc = UU::OS::REST::RemoteClient.new(Request)

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', request_uri)
          return Request::RequestAttributes.new(res)
        end
      end
      
      # Sets attributes of the specified request.
      #
      #
      # @param [String, UU::OS::UESURI] request_uri UESURI of the request whose attributes are about to be changed.
      # @param [RequestSetAttributes] request DTO containing the attributes of the request to be set.
      #
      # @return [UU::OS::UESURI] UESURI of the modified property.
      def set_attributes(request_uri, request = nil)
        svc = UU::OS::REST::RemoteClient.new(Request)
        payload = UU::Request::Request::RequestSetAttributes.new(request).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', request_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # This command deletes the specified request from the system. The request is
      # deleted even if it contains a locked sheet or attachment. The request
      # cannot be deleted when:
      #   the request's workflow is not created
      #   a property contains a reference to this request
      #   a time sheet exists with a reference to this request
      #
      # @param [UU::OS::UESURI,String] request_uri UESURI of the request which is about to be deleted.
      def delete(request_uri)
        svc = UU::OS::REST::RemoteClient.new(Request)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('delete', request_uri)
        end
      end
    end
  end
end